@@ -53,6 +53,7 @@ from polars.datatypes import (
     Binary,
     Boolean,
     Categorical,
+    Categories,
     DataType,
     Date,
     Datetime,
@@ -154,11 +155,14 @@ from polars.functions import (
     repeat,
     rolling_corr,
     rolling_cov,
+    row_index,
     select,
+    self_dtype,
     set_random_seed,
     sql_expr,
     std,
     struct,
+    struct_with_fields,
     sum,
     sum_horizontal,
     tail,
@@ -255,6 +259,7 @@ __all__ = [
     "Binary",
     "Boolean",
     "Categorical",
+    "Categories",
     "DataType",
     "Date",
     "Datetime",
@@ -337,6 +342,7 @@ __all__ = [
     "business_day_count",
     "concat",
     "dtype_of",
+    "struct_with_fields",
     "date_range",
     "date_ranges",
     "datetime_range",
@@ -344,6 +350,7 @@ __all__ = [
     "element",
     "ones",
     "repeat",
+    "self_dtype",
     "time_range",
     "time_ranges",
     "zeros",
@@ -410,6 +417,7 @@ __all__ = [
     "reduce",
     "rolling_corr",
     "rolling_cov",
+    "row_index",
     "select",
     "std",
     "struct",
