@@ -36,6 +36,10 @@ class ExprListNameSpace:
         """
         Evaluate whether all boolean values in a list are true.
 
+        Notes
+        -----
+        If there are no non-null elements in a row, the output is `True`.
+
         Examples
         --------
         >>> df = pl.DataFrame(
@@ -62,6 +66,10 @@ class ExprListNameSpace:
         """
         Evaluate whether any boolean value in a list is true.
 
+        Notes
+        -----
+        If there are no non-null elements in a row, the output is `False`.
+
         Examples
         --------
         >>> df = pl.DataFrame(
@@ -171,7 +179,7 @@ class ExprListNameSpace:
         │ ---       ┆ --- ┆ ---       │
         │ list[i64] ┆ i64 ┆ list[i64] │
         ╞═══════════╪═════╪═══════════╡
-        │ [1, 2, 3] ┆ 2   ┆ [2, 1]    │
+        │ [1, 2, 3] ┆ 2   ┆ [2, 3]    │
         │ [4, 5]    ┆ 1   ┆ [5]       │
         └───────────┴─────┴───────────┘
         """
@@ -196,6 +204,10 @@ class ExprListNameSpace:
         """
         Sum all the lists in the array.
 
+        Notes
+        -----
+        If there are no non-null elements in a row, the output is `0`.
+
         Examples
         --------
         >>> df = pl.DataFrame({"values": [[1], [2, 3]]})
